@@ -708,8 +708,6 @@ ADDITIONAL_DEFINITIONS = (
  *****************************************************************************/"""
     ,
     """
-extern char *text2cstring(const text *textptr);
-extern text *cstring2text(const char *cstring);
 """ + LIBLWGEOM_DEFINITIONS + """
 /*****************************************************************************
  * Type definitions
